
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.image.ImageObserver;

import java.util.Vector;

class MediaTrack implements ImageObserver
{
	MediaTracker tracker;
	int flags = 0;

	Image image;
	int id;
	int w;
	int h;

	public synchronized boolean imageUpdate(Image img, int status, int x, int y, int width, int height)
	{
		if ((status & WIDTH) != 0)
			w = width;

		if ((status & HEIGHT) != 0)
			h = height;

		if ((status & ABORT) != 0)
			flags |= MediaTracker.ABORTED;

		if ((status & ERROR) != 0)
			flags |= MediaTracker.ERRORED;

		if ((status & (FRAMEBITS | ALLBITS)) != 0)
			flags |= MediaTracker.COMPLETE;

		if ((status & (ABORT | ERROR | FRAMEBITS | ALLBITS)) != 0)
			flags &= ~MediaTracker.LOADING;

		boolean done = (flags & MediaTracker.LOADING) == 0;

		if (done)
			notifyAll();

		return done;
	}

	synchronized void waitFor(long tm) throws InterruptedException
	{
		if ((flags & MediaTracker.LOADING) != 0)
			wait(tm);
	}

	synchronized void waitFor() throws InterruptedException
	{
		if ((flags & MediaTracker.LOADING) != 0)
			wait();
	}

	void startLoad()
	{
		boolean done = (w > 0 || h > 0)
			? tracker.comp.prepareImage(image, w, h, this)
			: tracker.comp.prepareImage(image, this);

		flags |= done
			? MediaTracker.COMPLETE
			: MediaTracker.LOADING;
	}

	MediaTrack(MediaTracker tracker, Image image, int id, int w, int h)
	{
		this.tracker = tracker;
		this.image = image;
		this.id = id;
		this.w = w;
		this.h = h;
	}
}

public class MediaTracker
{
	public final static int LOADING		= 1;
	public final static int ABORTED		= 2;
	public final static int ERRORED		= 4;
	public final static int COMPLETE	= 8;

	Component comp;

	private Vector ids = new Vector();

	private Vector tracks(int id)
	{
		if (id >= ids.size())
			ids.ensureCapacity(id + 1);

		Vector tracks = (Vector) ids.elementAt(id);
		if (tracks == null)
		{
			tracks = new Vector();
			ids.setElementAt(tracks, id);
		}

		return tracks;
	}

	public void addImage(Image image, int id, int w, int h)
	{
		Vector set = tracks(id);
		set.addElement(new MediaTrack(this, image, id, w, h));
	}

	public void addImage(Image image, int id)
	{
		addImage(image, id, -1, -1);
	}

	public boolean checkID(int id, boolean load)
	{
		boolean done = true;

		Vector tracks = tracks(id);

		int n = tracks.size();
		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			if (load && track.flags == 0)
				track.startLoad();
			if ((track.flags & (ABORTED | ERRORED | COMPLETE)) != 0)
				continue;
			if ((track.flags & LOADING) != 0)
				done = false;
		}

		return done;
	}

	public boolean checkID(int id)
	{
		return checkID(id, false);
	}

	public boolean checkAll(boolean load)
	{
		boolean done = true;
		int n = ids.size();
		for (int i = 0; i < n; i++)
			if (!checkID(i, load))
				done = false;
		return done;
	}

	public boolean checkAll()
	{
		return checkAll(false);
	}

	public int statusID(int id, boolean load)
	{
		Vector tracks = tracks(id);
		int n = tracks.size();
		int flags = 0;

		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			if (load && track.flags == 0)
				track.startLoad();
			flags |= track.flags;
		}

		return flags;
	}

	public int statusAll(boolean load)
	{
		int flags = 0;
		int n = ids.size();
		for (int i = 0; i < n; i++)
			flags |= statusID(i, load);
		return flags;
	}

	public boolean isErrorID(int id)
	{
		Vector tracks = tracks(id);
		int n = tracks.size();

		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			if ((track.flags & ERRORED) != 0)
				return true;
		}

		return false;
	}

	public boolean isErrorAny()
	{
		int n = ids.size();
		for (int i = 0; i < n; i++)
			if (isErrorID(i))
				return true;
		return false;
	}

	public Object[] getErrorsID(int id)
	{
		Vector errors = new Vector();

		Vector tracks = tracks(id);
		int n = tracks.size();

		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			if ((track.flags & ERRORED) != 0)
				errors.addElement(track);
		}

		n = errors.size();
		Object[] result = new Object[n];
		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			result[i] = track.image;
		}

		return result;
	}

	public Object[] getErrorsAny()
	{
		Vector v = new Vector();
		int n = ids.size();
		for (int i = 0; i < n; i++)
		{
			Object[] resultID = getErrorsID(i);
			for (int j = 0; j < resultID.length; j++)
				v.addElement(resultID[j]);
		}

		Object[] result = new Object[v.size()];
		v.copyInto(result);
		return result;
	}

	public void waitForID(int id) throws InterruptedException
	{
		Vector tracks = tracks(id);
		int n = tracks.size();

		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			track.waitFor();
		}
	}

	public void waitForAll() throws InterruptedException
	{
		int n = ids.size();
		for (int i = 0; i < n; i++)
			waitForID(i);
	}

	public boolean waitForID(int id, long ms) throws InterruptedException
	{
		long stopTime = System.currentTimeMillis() + ms;

		Vector tracks = tracks(id);
		int n = tracks.size();

		for (int i = 0; i < n; i++)
		{
			MediaTrack track = (MediaTrack) tracks.elementAt(i);
			long delay = stopTime - System.currentTimeMillis();
			if (delay < 0)
				return false;
			track.waitFor(delay);
		}

		return true;
	}

	public boolean waitForAll(long ms) throws InterruptedException
	{
		long stopTime = System.currentTimeMillis() + ms;

		int n = ids.size();
		for (int i = 0; i < n; i++)
		{
			long delay = stopTime - System.currentTimeMillis();
			if (delay < 0)
				return false;
			waitForID(i, delay);
		}
		return true;
	}

	public MediaTracker(Component comp)
	{
		this.comp = comp;
	}
}

