
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class BufferedInputStream extends FilterInputStream
{ 
	protected byte[] buf; 
	protected int count;  // number of available bytes in buf (usually buf.length except near EOF.)
	protected int pos;    // current offset in the buffer for the next read

	protected int marklimit;
	protected int markpos = -1;

	private int refresh() throws IOException
	{
		pos = 0;
		markpos = -1;
		count = super.read(buf, 0, buf.length);
		return count;
	}

	public synchronized int available() throws IOException
	{
		// space in buf + underlying space in stream
		return count - pos + super.available();
	}

	public synchronized void mark(int readlimit)
	{
		if (buf.length < readlimit)
		{
			byte[] newbuf = new byte[readlimit];
			System.arraycopy(buf, pos, newbuf, 0, count);
			buf = newbuf;
			pos = 0;
		}
		else if (buf.length - count < readlimit)
		{
			System.arraycopy(buf, pos, buf, 0, count);
			pos = 0;
		}

		markpos = pos;
		marklimit = readlimit;
	}

	public boolean markSupported()
	{
		return true;
	}

	/**
	 * Read a byte of data from this stream, or -1 if
	 * the end of stream is reached.
	 *
	 * @return An integer between 0 and 255 if the read 
	 * is successfull, or -1 if the end of the stream
	 * was reached.
	 */
	public synchronized int read() throws IOException
	{
		if (count - pos <= 0)
			if (refresh() <= 0)
				return -1;

		return ((int)buf[pos++]) & 0xFF; // remove sign extension from byte
	}

	public synchronized int read(byte[] b, int offset, int len) throws IOException
	{
		int left = count - pos;

		if (len < left)
		{
			System.arraycopy(buf, pos, b, offset, len);
			pos += len;
			return len;
		}
		else
		{
			System.arraycopy(buf, pos, b, offset, left);
			pos = 0;
			count = 0;
			markpos = -1;
			int n = super.read(b, offset + left, len - left);
			return (n <= 0) ? left : left + n;
		}
	}

	public synchronized void reset() throws IOException
	{
		// complain if they didn't mark() the buffer
		if (markpos < 0)
			throw new IOException();

		pos = markpos;
	}

	public synchronized long skip(long n) throws IOException
	{
		if (n > Integer.MAX_VALUE)
		{
			throw new InternalError("BufferedInputStream.skip only handles int-sized jumps.  Fix me.");
		}

		int target = pos + (int) n;
		if (target < count)
		{
			pos = target;
			return n;
		}

		// skipping beyond end of buf

		int bufskip = count - pos; // bytes skipped in the buffer
		long fdskip; // bytes skipped in the underlying stream.

		pos = count;
		fdskip = super.skip(target - count);

		return bufskip + (int) fdskip;
	}

	public BufferedInputStream(InputStream in, int size)
	{
		super(in);
		buf = new byte[size];
	}

	public BufferedInputStream(InputStream in)
	{
		this(in, 512);
	}
}

