
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.io.NativeIO;

import java.util.Vector;

public class File
{
	public final static String pathSeparator = System.getProperty("path.separator");
	public final static char pathSeparatorChar = System.getProperty("path.separator").charAt(0);

	public final static String separator = System.getProperty("file.separator");
	public final static char separatorChar = System.getProperty("file.separator").charAt(0);

	private String path;

	// Generic (Object) Functions

	public boolean equals(Object obj)
	{
		if (!(obj instanceof File))
			return false;
		return ((File) obj).path.equals(path);
	}

	public int hashCode()
	{
		return path.hashCode();
	}

	public String toString()
	{
		return path;
	}

	// Pathname Functions

	public String getPath()
	{
		return path;
	}

	public boolean isAbsolute()
	{
		return	(path.charAt(0) == separatorChar) ||
			(Character.isLetter(path.charAt(0)) &&
			 (path.charAt(1) == ':') &&
			 (path.charAt(2) == '\\')
			);
	}

	public String getAbsolutePath()
	{
		return isAbsolute()
			? path
			: System.getProperty("user.dir") + separator + path;
	}

	public String getName()
	{
		int n = path.lastIndexOf(separatorChar);
		return (n < 0) ? path : path.substring(n + 1);
	}

	public String getParent()
	{
		int n = path.lastIndexOf(separatorChar);
		return (n < 0) ? null : path.substring(0, n);
	}

	// File Information (stat) Functions

	public boolean canRead()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return NativeIO.canRead(path);
	}

	public boolean canWrite()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkWrite(path);
		return NativeIO.canWrite(path);
	}

	public boolean exists()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return NativeIO.exists(path);
	}

	public boolean isDirectory()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return NativeIO.isDirectory(path);
	}

	public boolean isFile()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return NativeIO.isFile(path);
	}

	public long lastModified()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return (long) NativeIO.lastModified(path);
	}

	public long length()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return (long) NativeIO.fileSize(path);
	}

	// Directory Functions

	public String[] list()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		return NativeIO.dirList(path);
	}

	public String[] list(FilenameFilter filter)
	{
		String[] all = list();

		Vector v = new Vector();
		for (int i = 0; i < all.length; i++)
			if (filter.accept(this, all[i]))
				v.addElement(all[i]);

		String[] result = new String[v.size()];
		for (int i = 0; i < result.length; i++)
			result[i] = (String) v.elementAt(i);

		return result;
	}

	// File Manipulation Functions

	public boolean delete()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkDelete(path);
		return NativeIO.delete(path);
	}

	public boolean renameTo(File dest)
	{
		String destPath = dest.getPath();
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkWrite(destPath);
		return NativeIO.rename(path, destPath);
	}

	public boolean mkdir()
	{
		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkWrite(path);
		return NativeIO.mkdir(path);
	}

	public boolean mkdirs()
	{
		File parent = new File(getParent());
		if (!parent.exists())
			if (!parent.mkdirs())
				return false;
		return mkdir();
	}


	public File(String path)
	{
		kore.util.NativeDebug.printStrln(kore.util.NativeDebug.FILE,
						 "File(path=" +path+ ")");

		this.path = path;
	}

	public File(File dir, String name)
	{
		kore.util.NativeDebug.printStrln(kore.util.NativeDebug.FILE,
						 "File(dir=" +((dir==null)?"null":dir.getPath())+ 
						 "; name=" +name+ ")");

		this.path =
			(dir == null)
			? name
			: dir.getPath() + separator + name;
	}

	public File(String path, String name)
	{
		kore.util.NativeDebug.printStrln(kore.util.NativeDebug.FILE,
						 "File(path=" +path+ "; name=" +name+ ")");

		if (path != null)
			this.path = path + separator + name;
		else
			this.path = name;
	}
}

