
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

import java.io.NativeIO;

class NonClosingFileInputStream extends FileInputStream
{
	public synchronized void close() throws IOException
	{
	}

	public NonClosingFileInputStream(FileDescriptor fd)
	{
		super(fd);
	}
}

class NonClosingFileOutputStream extends FileOutputStream
{
	public synchronized void close() throws IOException
	{
	}

	public NonClosingFileOutputStream(FileDescriptor fd)
	{
		super(fd);
	}
}

class PushbackFileInputStream extends PushbackInputStream
{
	private FileDescriptor fd;

	public synchronized int available() throws IOException
	{
		return in.available();
	}

	public synchronized int read() throws IOException
	{
		return in.read();
	}

	public synchronized int read(byte[] b, int off, int len) throws IOException
	{
		return in.read(b, off, len);
	}

	public synchronized void unread(int c) throws IOException
	{
		if (fd == null)
			throw new IOException();
		NativeIO.fseek(fd, -1, NativeIO.SEEK_CUR);
	}

	public PushbackFileInputStream(InputStream in, FileDescriptor fd)
	{
		super(in);
		this.fd = fd;
	}
}

public class RandomAccessFile implements DataInput, DataOutput
{
	private FileDescriptor fd;

	private DataInputStream din;
	private DataOutputStream dout;

	private volatile boolean closing;

	public final FileDescriptor getFD() throws IOException
	{
		return fd;
	}

	public synchronized void close() throws IOException
	{
		if (closing)
			return;
		closing = true;

		if (fd == null)
			throw new IOException();
		dout.close();
		din.close();
		NativeIO.close(fd);
		fd = null;

		closing = false;
	}

	public long getFilePointer() throws IOException
	{
		if (fd == null)
			throw new IOException();
		return (long) NativeIO.ftell(fd);
	}

	public void seek(long pos) throws IOException
	{
		if (fd == null)
			throw new IOException();
		NativeIO.fseek(fd, (int) pos, NativeIO.SEEK_SET);
	}

	public long length() throws IOException
	{
		if (fd == null)
			throw new IOException();
		long pos = getFilePointer();
		long length = (long) NativeIO.fseek(fd, 0, NativeIO.SEEK_END);
		seek(pos);
		return length;
	}

	/**
	 * skip <code>n</code> bytes in the file, if possible.
	 *
	 * @param n the number of bytes to skip.
	 * @return the number of bytes actually skipped.
	 */
	public synchronized int skipBytes(int n) throws IOException
	{
		if (fd == null)
			throw new IOException();
		int pos = (int) getFilePointer();
		return NativeIO.fseek(fd, n, NativeIO.SEEK_CUR) - pos;
	}

	public int read() throws IOException
	{
		if (fd == null)
			throw new IOException();
		byte[] b = new byte[1];
		if (read(b, 0, 1) < 0)
			return -1;
		return ((int)b[0]) & 0xFF; 
	}

	public int read(byte[] b) throws IOException
	{
		return read(b, 0, b.length);
	}

	public int read(byte[] b, int off, int len) throws IOException
	{
		return NativeIO.read(fd, b, off, len);
	}

	public void write(int c) throws IOException
	{
		byte[] b = new byte[1];
		b[0] = (byte) c;
		write(b);
	}

	public void write(byte[] b) throws IOException
	{
		write(b, 0, b.length);
	}

	public void write(byte[] b, int off, int len) throws IOException
	{
		NativeIO.write(fd, b, off, len);
	}


	public final boolean readBoolean() throws IOException
	{
		return din.readBoolean();
	}

	public final byte readByte() throws IOException
	{
		return din.readByte();
	}

	public final int readUnsignedByte() throws IOException
	{
		return din.readUnsignedByte();
	}

	public final char readChar() throws IOException
	{
		return din.readChar();
	}

	public final short readShort() throws IOException
	{
		return din.readShort();
	}

	public final int readUnsignedShort() throws IOException
	{
		return din.readUnsignedShort();
	}

	public final int readInt() throws IOException
	{
		return din.readInt();
	}

	public final long readLong() throws IOException
	{
		return din.readLong();
	}

	public final float readFloat() throws IOException
	{
		return din.readFloat();
	}

	public final double readDouble() throws IOException
	{
		return din.readDouble();
	}

	public final String readLine() throws IOException
	{
		return din.readLine();
	}

	public final String readUTF() throws IOException
	{
		return din.readUTF();
	}

	public final void readFully(byte[] b) throws IOException
	{
		din.readFully(b);
	}

	public final void readFully(byte[] b, int off, int len) throws IOException
	{
		din.readFully(b, off, len);
	}


	public final void writeBoolean(boolean v) throws IOException
	{
		dout.writeBoolean(v);
	}

	public final void writeByte(int v) throws IOException
	{
		dout.writeByte(v);
	}

	public final void writeChar(int v) throws IOException
	{
		dout.writeChar(v);
	}

	public final void writeShort(int v) throws IOException
	{
		dout.writeShort(v);
	}

	public final void writeInt(int v) throws IOException
	{
		dout.writeInt(v);
	}

	public final void writeLong(long v) throws IOException
	{
		dout.writeLong(v);
	}

	public final void writeFloat(float v) throws IOException
	{
		dout.writeFloat(v);
	}

	public final void writeDouble(double v) throws IOException
	{
		dout.writeDouble(v);
	}

	public final void writeBytes(String s) throws IOException
	{
		dout.writeBytes(s);
	}

	public final void writeChars(String s) throws IOException
	{
		dout.writeChars(s);
	}

	public final void writeUTF(String s) throws IOException
	{
		dout.writeUTF(s);
	}


	public RandomAccessFile(String path, String mode) throws IOException
	{
		if (!mode.equals("r") && !mode.equals("rw"))
			throw new IllegalArgumentException();

		SecurityManager s = System.getSecurityManager();
		if (s != null)
			s.checkRead(path);
		if (s != null && mode.equals("rw"))
			s.checkWrite(path);

		fd = new FileDescriptor();
		if (mode.equals("rw"))
			NativeIO.open_read_write(fd, path);
		else
			NativeIO.open_read(fd, path);

		InputStream in = new NonClosingFileInputStream(fd);
		InputStream pin = new PushbackFileInputStream(in, fd);
		din = new DataInputStream(in);

		OutputStream out = new NonClosingFileOutputStream(fd);
		dout = new DataOutputStream(out);
	}

	public RandomAccessFile(File file, String mode) throws IOException
	{
		this(file.getPath(), mode);
	}
}

