
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.io;

public class StreamTokenizer
{ 
	private PushbackInputStream in;
	private int line;

	private int[] type = new int[0x100];

	private final static byte[] numericChars = {
		'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.', '-'
	};

	private final static int ORDINARY	= 0;
	private final static int COMMENT	= 1;
	private final static int WORD		= 2;
	private final static int QUOTE		= 3;
	private final static int WHITESPACE	= 4;
	private final static int NUMERIC	= 5;

	private boolean eolSignificant;
	private boolean lowerCase;
	private boolean slashSlashCommentsEnabled;
	private boolean slashStarCommentsEnabled;

	private boolean pushback;


	public double nval;
	public String sval;
	public int ttype;

	public final static int TT_EOF		= -1;
	public final static int TT_EOL		= '\n';
	public final static int TT_NUMBER	= -2;
	public final static int TT_WORD		= -3;

	public void ordinaryChars(int lo, int hi)
	{
		for (int i = lo; i <= hi; i++)
			type[i] = ORDINARY;
	}

	public void ordinaryChar(int c)
	{
		ordinaryChars(c, c);
	}

	public void resetSyntax()
	{
		ordinaryChars(0, 0xFF);
	}

	public void commentChar(int c)
	{
		type[c] = COMMENT;
	}

	public void quoteChar(int c)
	{
		type[c] = QUOTE;
	}

	public void parseNumbers()
	{
		for (int i = 0; i < numericChars.length; i++)
			type[numericChars[i]] = NUMERIC;
	}

	public void whitespaceChars(int lo, int hi)
	{
		for (int i = lo; i <= hi; i++)
			type[i] = WHITESPACE;
	}

	public void wordChars(int lo, int hi)
	{
		for (int i = lo; i <= hi; i++)
			type[i] = WORD;
	}

	public void slashSlashComments(boolean flag)
	{
		slashSlashCommentsEnabled = flag;
	}

	public void slashStarComments(boolean flag)
	{
		slashStarCommentsEnabled = flag;
	}

	public void eolIsSignificant(boolean flag)
	{
		eolSignificant = flag;
	}

	public void lowerCaseMode(boolean flag)
	{
		lowerCase = flag;
	}

	public int lineno()
	{
		return line;
	}

	public void pushBack()
	{
		pushback = true;
	}

	public int nextToken() throws IOException
	{
		if (pushback)
		{
			pushback = false;
			return ttype;
		}

		StringBuffer buff = new StringBuffer();
		int c;

		while (true)
		{
			c = in.read();
			if (c == -1)
			{
				ttype = TT_EOF;
				return ttype;
			}

			if (c == '/')
			{
				c = in.read();
				if (c == '/' && slashSlashCommentsEnabled)
				{
					while (c != '\n')
						c = in.read();
					continue;
				}
				else if (c == '*' && slashStarCommentsEnabled)
				{
					while (c != '/')
						while (c != '*')
							c = in.read();
					continue;
				}
				else
				{
					in.unread(c);
					c = '/';
				}
			}

			if (c == '\n' && eolSignificant)
			{
				ttype = TT_EOL;
				return ttype;
			}

			switch (type[c])
			{
			case WHITESPACE:
				break;

			case COMMENT:
				while (c != '\n')
					c = in.read();
				break;

			case QUOTE:
				ttype = c;

				while (true)
				{
					c = in.read();
					if (c == ttype || c == '\n')
					{
						sval = buff.toString();
						return ttype;
					}
					else
						buff.append((char) c);
				}

			case WORD:
				while (true)
				{
					c = in.read();
					switch (type[c])
					{
					case WORD:
					case NUMERIC:
						buff.append((char) c);
						break;

					default:
						in.unread(c);
						sval = buff.toString();
						if (lowerCase)
							sval = sval.toLowerCase();
						ttype = TT_WORD;
						return ttype;
					}
				}


			case NUMERIC:
				while (true)
				{
					c = in.read();
					if (c < 0 || type[c] != NUMERIC)
					{
						in.unread(c);
						sval = buff.toString();
						try
						{
							nval = Double.valueOf(sval).doubleValue();
							ttype = TT_NUMBER;
							return ttype;
						}
						catch (NumberFormatException e)
						{
							if (lowerCase)
								sval = sval.toLowerCase();
							ttype = TT_WORD;
							return ttype;
						}
					}
					else
						buff.append((char) c);
				}
			}
		}
	}

	public String toString()
	{
		switch (ttype)
		{
		case TT_EOF:	return "EOF";
		case TT_EOL:	return "\n";
		case TT_NUMBER:	return Double.toString(nval);
		case TT_WORD:	return sval;
		default:	return super.toString();
		}
	}

	public StreamTokenizer(InputStream in)
	{
		this.in = new PushbackInputStream(in);

		wordChars('A', 'Z');
		wordChars('a', 'z');
		wordChars('\u00A0', '\u00FF');
		whitespaceChars('\u0000', '\u0020');
		commentChar('/');
		quoteChar('\'');
		quoteChar('"');
		parseNumbers();
		eolIsSignificant(false);
		slashSlashComments(false);
		slashStarComments(false);
	}
}
