// java.lang.Double
// (c) Peter Nagy
// API version: 1.0.2
// History:
//  1997-01-18 Initial version	Peter Nagy
//  1997-02-25 modified		Glynn Clements <glynn@sensei.co.uk>
//  1997-12-02 modified		Glynn Clements <glynn@sensei.co.uk>
//	valueOf() written to use DoubleParser

package java.lang;

import kore.lang.DoubleParser;

public final class Double extends Number
{
	public static final double MIN_VALUE = 5e-324;
	public static final double MAX_VALUE = 1.7976931348623157e+308;
	public static final double NEGATIVE_INFINITY = -1.0d/0.0d;
	public static final double POSITIVE_INFINITY = 1.0d/0.0d;
	public static final double NaN = 0.0d/0.0d;

	private double value;

	public static Double valueOf(String s) throws NumberFormatException
	{
		DoubleParser parser = new DoubleParser(s);
		return new Double(parser.parse());
	}

	public static String toString(double v)
	{
		if (v == NaN)
			return "NaN";
		if (v == POSITIVE_INFINITY)
			return "Infinity";
		if (v == NEGATIVE_INFINITY)
			return "-Infinity";

		boolean negative = (doubleToLongBits(v) & (1L << 63)) != 0;
		double m = negative ? -v : v;

		if (m == 0.0d)
			return negative ? "-0.0" : "0.0";

		String result;

		if (m >= 1e-3 && m < 1e7)
		{
			result = "" + (int) m + ".";
			while (m > 0.0d)
			{
				m *= 10;
				int digit = (int) m;
				m -= digit;
				result += digit;
			}
		}
		else
		{
			int exponent = (int) (Math.log(m) / Math.log(10.0d));
			double mantissa = m / Math.pow(10.0d, (double) exponent);
			result = toString(mantissa) + "E" + Integer.toString(exponent);
		}

		return negative ? ("-" + result) : result;
	}

	public static long doubleToLongBits(double value)
	{
		if (value == NaN)
			return 0x7FF8000000000000L;
		if (value == POSITIVE_INFINITY)
			return 0x7FF0000000000000L;
		if (value == NEGATIVE_INFINITY)
			return 0xFFF0000000000000L;

		boolean sign = value < 0.0d;	// TODO: make this work with -0.0d
		double magnitude = sign ? -value : value;
		int exponent = (int) Math.ceil(Math.log(magnitude) / Math.log(2.0d)) + 1075;
		double mantissa = magnitude / Math.pow(2.0d, (double) exponent);

		long bits = sign ? 0x8000000000000000L : 0L;
		bits |= exponent << 52;
		bits |= (long) (mantissa * 0x000FFFFFFFFFFFFFL);

		return bits;
	}

	public static double longBitsToDouble(long bits)
	{
		if (bits == 0x7F80000000000000L)
			return POSITIVE_INFINITY;
		if (bits == 0xFF80000000000000L)
			return NEGATIVE_INFINITY;
		if (bits >= 0x7FF0000000000001L && bits <= 0x7FFFFFFFFFFFFFFFL)
			return NaN;
		if (bits >= 0xFFF0000000000001L && bits <= 0xFFFFFFFFFFFFFFFFL)
			return NaN;

		int sign = ((bits >> 63) == 0) ? 1 : -1;
		int exponent = (int)((bits >> 52) & 0x7FFL);
		long mantissa = bits & 0x000FFFFFFFFFFFFFL;
		if (exponent == 0)
			mantissa <<= 1;
		else
			mantissa |= 0x0010000000000000L;

		return sign * mantissa * Math.pow(2.0d, (double) (exponent - 1075));
	}

	public static boolean isInfinite(double v)
	{
		return (v == POSITIVE_INFINITY || v == NEGATIVE_INFINITY);
	}

	public static boolean isNaN(double v)
	{
		return v != v;
	}

	public String toString()
	{
		return toString(value);
	}

	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;

		if (!(obj instanceof Double))
			return false;

		Double d = (Double) obj;

		return value == d.doubleValue();
	}

	public int hashCode()
	{
		long v = doubleToLongBits(value);
		return (int) (v ^ (v >>> 32));
	}

	public int intValue()
	{
		return (int) value;
	}

	public long longValue()
	{
		return (long) value;
	}

	public float floatValue()
	{
		return (float) value;
	}

	public double doubleValue()
	{
		return value;
	}

	public boolean isInfinite()
	{
		return isInfinite(value);
	}

	public boolean isNaN()
	{
		return isNaN(value);
	}

	public Double(double value)
	{
		this.value = value;
	}

	public Double(String s) throws NumberFormatException
	{
		this.value = valueOf(s).doubleValue();
	}
}

