// java.lang.Float
// (c) Peter Nagy
// API version: 1.0.2
// History:
//  1997-01-18 Initial version	Peter Nagy
//  1997-02-25 modified		Glynn Clements <glynn@sensei.co.uk>


package java.lang;

public final class Float extends Number
{
	public static final float MAX_VALUE = 3.4028235e+38f;
	public static final float MIN_VALUE = 1.4e-45f;
	public static final float NEGATIVE_INFINITY = -1.0f/1.0f;
	public static final float POSITIVE_INFINITY = 1.0f/1.0f;
	public static final float NaN = 0.0f/0.0f;

	private float value;

	public static Float valueOf(String s) throws NumberFormatException
	{
		return new Float(Double.valueOf(s).floatValue());
	}

	public static String toString(float v)
	{
		if (v == NaN)
			return "NaN";
		if (v == POSITIVE_INFINITY)
			return "Infinity";
		if (v == NEGATIVE_INFINITY)
			return "-Infinity";

		boolean negative = (floatToIntBits(v) & (1 << 31)) != 0;
		float m = negative ? -v : v;

		if (m == 0.0f)
			return negative ? "-0.0" : "0.0";

		String result;

		if (m >= 1e-3 && m < 1e7)
		{
			result = "" + (int) m + ".";
			while (m > 0.0d)
			{
				m *= 10;
				int digit = (int) m;
				m -= digit;
				result += digit;
			}
		}
		else
		{
			int exponent = (int) (Math.log(m) / Math.log(10.0d));
			float mantissa = (float) (m / Math.pow(10.0d, (double) exponent));
			result = toString(mantissa) + "E" + Integer.toString(exponent);
		}

		return negative ? ("-" + result) : result;
	}

	public static int floatToIntBits(float value)
	{
		if (value == NaN)
			return 0x7FC00000;
		if (value == POSITIVE_INFINITY)
			return 0x7F800000;
		if (value == NEGATIVE_INFINITY)
			return 0xFF800000;

		boolean sign = value < 0.0;	// TODO: make this work with -0.0
		float magnitude = sign ? -value : value;
		int exponent = (int) Math.ceil(Math.log(magnitude) / Math.log(2.0d)) + 150;
		float mantissa = magnitude / (float) Math.pow(2.0d, (double) exponent);

		int bits = sign ? 0x80000000 : 0;
		bits |= exponent << 23;
		bits |= (int) (mantissa * 0x007FFFFF);

		return bits;
	}

	public static float intBitsToFloat(int bits)
	{
		if (bits == 0x7F800000)
			return POSITIVE_INFINITY;
		if (bits == 0xFF800000)
			return NEGATIVE_INFINITY;
		if (bits >= 0x7F800001 && bits <= 0x7FFFFFFF)
			return NaN;
		if (bits >= 0xFF800001 && bits <= 0xFFFFFFFF)
			return NaN;

		int sign = ((bits >> 31) == 0) ? 1 : -1;
		int exponent = (int)((bits >> 52) & 0xFF);
		int mantissa = bits & 0x007FFFFF;
		if (exponent == 0)
			mantissa <<= 1;
		else
			mantissa |= 0x800000;

		return (float) (sign * mantissa * Math.pow(2.0d, (double) (exponent - 150)));
	}

	public static boolean isInfinite(float v)
	{
		return (v == POSITIVE_INFINITY || v == NEGATIVE_INFINITY);
	}

	public static boolean isNaN(float v)
	{
		return v != v;
	}

	public String toString()
	{
		return toString(value);
	}

	public boolean equals(Object obj)
	{
		if (obj == null)
			return false;

		if (!(obj instanceof Float))
			return false;

		Float f = (Float) obj;

		return value == f.floatValue();
	}

	public int hashCode()
	{
		return floatToIntBits(value);
	}

	public int intValue()
	{
		return (int) value;
	}

	public long longValue()
	{
		return (long) value;
	}

	public float floatValue()
	{
		return (float) value;
	}

	public double doubleValue()
	{
		return (double) value;
	}

	public boolean isInfinite()
	{
		return isInfinite(value);
	}

	public boolean isNaN()
	{
		return isNaN(value);
	}

	public Float(float value)
	{
		this.value = value;
	}

	public Float(double value)
	{
		this.value = (float)value;
	}

	public Float(String s) throws NumberFormatException
	{
		this.value = valueOf(s).floatValue();
	}
}

