
// (C) 1996 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.lang;

import java.lang.NativeLang;

import java.io.InputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileDescriptor;
import java.io.PrintStream;
import java.io.IOException;

import java.util.Properties;

public final class System
{
	public static InputStream in;
	public static PrintStream out;
	public static PrintStream err;

	private static SecurityManager sm;
	private static Properties properties;

	static
	{
		in = new FileInputStream(FileDescriptor.in);
		out = new PrintStream(new FileOutputStream(FileDescriptor.out));
		err = new PrintStream(new FileOutputStream(FileDescriptor.err));
	}

	private static void initProperties()
	{
		int index;
		
		properties = new Properties();
		
		/* Load basic, VM-specific properties. */
		String[] VMprops = NativeLang.getVMProperties();
		if (VMprops != null) {
			for (index = 0;
			     index < (VMprops.length - 1);
			     //    -1 takes care of odd length array
			     index +=2) {
				properties.put(VMprops[index],
					       VMprops[index + 1]);
			}
		}
		VMprops = null;

		/* Now load any properties from the user's property file. */
		try
		{
			FileInputStream propIn = new FileInputStream(NativeLang.defaultPropertiesFile());
			properties.load(propIn);
			propIn.close();
		}
		catch (IOException e)
		{
			// Not having a properties file is okay.
		}

		/*
		 * Now load any properties the user defined on the
		 * command line.  Do these last so they correctly
		 * override both built-in properties and properties
		 * from the user's file.
		 */
		String[] clProperties = NativeLang.getCommandLineProperties();
		if (clProperties != null) {
			for (index = 0;
			     index < (clProperties.length - 1);
			     //   -1 takes care of odd length array
			     index += 2) {
				properties.put(clProperties[index],
					       clProperties[index+1]);
			}
		}
		clProperties = null;
	}

	/**
	 * A private method used only by the VM as a bootstrap point into
	 * the Java libraries.
	 * <p>
	 * This function will be called after the initial thread group and
	 * thread are created, but before they go off and do their own thing.
	 * It can do whatever is necessary to initialize the system.
	 */
	private static void
	initializeSystemClass()
	{
		if (properties == null) /* should always be null for this func. */
			initProperties();


		// Initialize Kore debugging stuff.
		String debugStr = properties.getProperty("kore.debug");
		kore.util.NativeDebug.setDebugMask(debugStr);
	}
	

	public static void setProperties(Properties props)
	{
		if (sm != null)
			sm.checkPropertiesAccess();
		properties = props;
	}

	public static Properties getProperties()
	{
		if (sm != null)
			sm.checkPropertiesAccess();
		if (properties == null)
			initProperties();
		return properties;
	}

	public static String getProperty(String key)
	{
		if (sm != null)
			sm.checkPropertyAccess(key);
		if (properties == null)
			initProperties();
		return properties.getProperty(key);
	}

	public static String getProperty(String key, String def)
	{
		if (sm != null)
			sm.checkPropertyAccess(key);
		if (properties == null)
			initProperties();
		return properties.getProperty(key, def);
	}

	public static SecurityManager getSecurityManager()
	{
		return sm;
	}

	public static void setSecurityManager(SecurityManager s)
	{
		if (sm != null)
			throw new SecurityException("SecurityManager already installed");
		sm = s;
	}

	public static long currentTimeMillis()
	{
		return NativeLang.currentTimeMillis();
	}

	public static void exit(int status)
	{
		Runtime.getRuntime().exit(status);
	}

	public static void gc()
	{
		Runtime.getRuntime().gc();
	}

	public static void load(String filename)
	{
		Runtime.getRuntime().load(filename);
	}

	public static void loadLibrary(String libname)
	{
		Runtime.getRuntime().loadLibrary(libname);
	}

	public static void arraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length)
	{
		String srcClass = src.getClass().getName();
		String dstClass = dst.getClass().getName();

		/*
		 * XXX charAt(0) and charAt(1) might throw ArrayIndexOutOfBounds, which would
		 * be the *Wrong* arrayindexoutofbounds message.
		 */

		if (srcClass.charAt(0) != '[')
			throw new ArrayStoreException("source is not an array");
		if (dstClass.charAt(0) != '[')
			throw new ArrayStoreException("destination is not an array");

		char srcType = srcClass.charAt(1);
		char dstType = dstClass.charAt(1);

		if (srcType == '[')
			srcType = 'L';

		if (dstType == '[')
			dstType = 'L';

		if (srcType != dstType)
			throw new ArrayStoreException("source and destination types differ");

		switch (srcType)
		{
		case 'Z':
			boolean[] zs = (boolean[]) src;
			boolean[] zd = (boolean[]) dst;
			for (int i = 0; i < length; i++)
				zd[dstOffset + i] = zs[srcOffset + i];
			break;

		case 'B':
			byte[] bs = (byte[]) src;
			byte[] bd = (byte[]) dst;
			for (int i = 0; i < length; i++)
				bd[dstOffset + i] = bs[srcOffset + i];
			break;

		case 'C':
			char[] cs = (char[]) src;
			char[] cd = (char[]) dst;
			for (int i = 0; i < length; i++)
				cd[dstOffset + i] = cs[srcOffset + i];
			break;

		case 'S':
			short[] ss = (short[]) src;
			short[] sd = (short[]) dst;
			for (int i = 0; i < length; i++)
				sd[srcOffset + i] = ss[dstOffset + i];
			break;

		case 'I':
			int[] is = (int[]) src;
			int[] id = (int[]) dst;
			for (int i = 0; i < length; i++)
				id[dstOffset + i] = is[srcOffset + i];
			break;

		case 'J':
			long[] ls = (long[]) src;
			long[] ld = (long[]) dst;
			for (int i = 0; i < length; i++)
				ld[dstOffset + i] = ls[srcOffset + i];
			break;

		case 'F':
			float[] fs = (float[]) src;
			float[] fd = (float[]) dst;
			for (int i = 0; i < length; i++)
				fd[dstOffset + i] = fs[srcOffset + i];
			break;

		case 'D':
			double[] ds = (double[]) src;
			double[] dd = (double[]) dst;
			for (int i = 0; i < length; i++)
				dd[dstOffset + i] = ds[srcOffset + i];
			break;

		case 'L':
			Object[] rs = (Object[]) src;
			Object[] rd = (Object[]) dst;
			for (int i = 0; i < length; i++)
				rd[dstOffset + i] = rs[srcOffset + i];
			break;
		}
	}
}

