
// (C) 1996 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.lang;

import java.lang.NativeLang;

/**
 * Kore implementation of the <code>java.lang.Thread</code> class.
 * <p>
 * All native methods are indirected through <code>java.lang.NativeLang</code>.
 *
 * @see java.lang.NativeLang
 * @version Kore 0.0.3, June 1997
 * @author Glynn Clements <a href="mailto:glynn@sensei.co.uk">glynn@sensei.co.uk</a>
 */
public class Thread implements Runnable
{
	public final static int MIN_PRIORITY	= 1;
	public final static int NORM_PRIORITY	= 5;
	public final static int MAX_PRIORITY	= 10;

	/**
	 * Number of threads created.  Used only for generating
	 * "unique" names.
	 *
	 * @see java.lang.Thread#autoName()
	 */
	private static int count = 0;

	/**
	 * Generate a "unique" default name for a thread.
	 */
	private static synchronized String autoName()
	{
		return "Thread-" + (++count);
	}

	private ThreadGroup group;
	private Runnable target;
	private String name;
	private int priority;
	private boolean daemon;

	private final static int CREATED	= 0;
	private final static int RUNNING	= 1;
	private final static int SUSPENDED	= 2;
	private final static int ASLEEP		= 3;
	private final static int INTERRUPTED	= 4;
	private final static int STOPPED	= 5;
	private final static int DESTROYED	= 6;
	private final static int WAITING	= 7;
	
	/**
	 * State is set to CREATED by the static initializer.
	 * Once set to other than CREATED, it should never go back.
	 * Alternates between RUNNING and SUSPENDED/WAITING as
	 * suspend()/wait() and reseume() are called.
	 * <p>
	 * Can be set to INTERRUPTED if ASLEEP, or WAITING.
	 * Can be set to DESTROYED at any time.
	 * Can be set to STOPPED at any time.
	 * <p>
	 * If a Thread blocks in a wait(), its state stays
	 * as RUNNING.  This might need to change.
	 * <p>
	 * Can sleep if RUNNING.
	 * <blockquote><pre>
	 *            /--------------------\
	 *            V                    |
	 * CREATED -> RUNNING -> SUSPENDED /
	 * </blockquote></pre>
	 *
	 * @see #setState(int)
	 * @see java.lang.NativeLang#setThreadState(Thread, int)
	 */
	private int state = CREATED;

        /**
	 * Used only by the <a href="http://www.kaffe.org/">Kaffe</a> VM internals.
	 */
    	private int PrivateInfo;

        /**
	 * Used only by the <a href="http://www.kaffe.org/">Kaffe</a> VM internals.
	 */
    	private Thread threadQ;

	public static Thread currentThread()
	{
		return NativeLang.currentThread();
	}

	public static boolean interrupted()
	{
		return currentThread().isInterrupted();
	}

	public static int activeCount()
	{
		return currentThread().getThreadGroup().activeCount();
	}

	public static int enumerate(Thread[] threads)
	{
		return currentThread().getThreadGroup().enumerate(threads);
	}

	public static void dumpStack()
	{
		new Exception("Stack trace").printStackTrace();
	}

	/*
	 * XXX needs to be synchronized to avoid losing interrupts?
	 * it checks and sets state, which should be protected...
	 */
	public static void sleep(long millis, int nanos) throws InterruptedException
	{
		NativeLang.sleep(millis, nanos);
	}

	public static void sleep(long millis) throws InterruptedException
	{
		sleep(millis, 0);
	}

	public static void yield()
	{
		NativeLang.yield();
	}

	/* XXX need to be synchronized? */
	private synchronized void setState(int state)
	{
		/*
		 * NativeLang needs to see current state and act
		 * accordingly, so it will update thread's state.
		 */
		NativeLang.setThreadState(this, state);
	}

	public String toString()
	{
		return	"Thread[" +
			name + "," +
			priority + "," +
			group.getName() +
			"]";
	}

	public void checkAccess()
	{
		SecurityManager sm = System.getSecurityManager();
		if (sm != null)
			sm.checkAccess(this);
	}

	/**
	 * Returns <code>true</code> if the thread represented by this
	 * object is running (including suspended, asleep, or interrupted.)
	 * Returns <code>false</code> if the thread hasn't be started yet,
	 * is stopped or destroyed.
	 *
	 * @return <code>true</code> if thread is alive, <code>false</code> if not.
	 * @see #start()
	 * @see #stop()
	 * @see #suspend()
	 * @see #interrupt()
	 */
	public final boolean isAlive()
	{
		switch (state)
		{
		case CREATED:
			return false;
		case RUNNING:
		case SUSPENDED:
		case WAITING:
		case ASLEEP:
		case INTERRUPTED:
			return true;
		case STOPPED:
		case DESTROYED:
			return false;
		default:
			throw new RuntimeException("reality failure");
		}
	}

	public int countStackFrames()
	{
		return NativeLang.countStackFrames(this);
	}

	/**
	 * Start this thread object.  After calling this method there will be
	 * a new active thread in the system (assuming things go well.)  The
	 * <code>Runnable</code> object, or the Thread's <code>run()</code>
	 * method will be started in a new thread.
	 *
	 */
	public void start()
	{
		switch (state)
		{
		case CREATED:
			setState(RUNNING);
			break;
		case RUNNING:
		case SUSPENDED:
		case WAITING:
		case ASLEEP:
		case INTERRUPTED:
			throw new IllegalThreadStateException("already started");
		case STOPPED:
			/* XXX */
			break;
		case DESTROYED:
			throw new IllegalThreadStateException("destroyed");
		}
	}

	public final void suspend()
	{
		checkAccess();
		if (state != RUNNING)
			return;
		setState(SUSPENDED);
	}

	public final void resume()
	{
		checkAccess();
		if (state != SUSPENDED)
			return;
		setState(RUNNING);
	}

	/**
	 * Join with the current thread.   The calling thread
	 * will block until this thread dies.
	 *
	 * XXX wait()s on the thread.  Seems like this would
	 * cause a spurious wakeup if some code synchronizes
	 * on the thread and uses wait/notify for some
	 * other purpose.
	 */
	public final synchronized void join() throws InterruptedException
	{
		while (isAlive()) 
		{
			/* wait sets this.state = WAITING;*/
			wait();
			/* wait sets this.state = RUNNING;*/
		}
	}

	public final synchronized void join(long millis) throws InterruptedException
	{
		if (millis == 0)
		{
			join();
			return;
		}

		long stopTime = System.currentTimeMillis() + millis;
		while (isAlive() && (millis > 0))
		{
			/* wait sets this.state = WAITING;*/
			wait(millis);
			millis = stopTime - System.currentTimeMillis();
			/* wait sets this.state = RUNNING;*/
		}
	}

	public final void join(long millis, int nanos) throws InterruptedException
	{
		join(millis); 
	}

	public void interrupt()
	{
		setState(INTERRUPTED);
	}

	public boolean isInterrupted()
	{
		return state == INTERRUPTED;
	}

	/**
	 * The polite way to kill a thread.
	 *
	 * @param error <code>Throwable</code> to toss in stopped thread.
	 */
	public final void stop(Throwable error)
	{
		checkAccess();

		if (error == null)
			throw new NullPointerException("Throwable is null");
		
		/* XXX synchronization? */
		NativeLang.stopThread(this, error);
	}

	public final void stop()
	{
		stop(new ThreadDeath());
	}

	/**
	 * Destroy the this thread.  Very abrupt.  Does <em>not</em>
	 * guarantee any cleanup is done on the native side.
	 */
	public void destroy()
	{
		setState(DESTROYED);
	}

	/**
	 * Method called only by the VM to let the object do some
	 * java-level cleanup.  Only here currently because
	 * <a href="http://www.kaffe.org">Kaffe</a> calls it.
	 */
	private void exit()
	{
	}

	public final String getName()
	{
		return name;
	}

	public final ThreadGroup getThreadGroup()
	{
		return isAlive() ? group : null;
	}

	public final int getPriority()
	{
		return priority;
	}

	public final boolean isDaemon()
	{
		return daemon;
	}

	public final void setDaemon(boolean daemon)
	{
		checkAccess();
		if (isAlive())
			throw new IllegalThreadStateException("already started");
		this.daemon = daemon;
	}

	public final void setName(String name)
	{
		checkAccess();
		this.name = name;
	}

	public final void setPriority(int priority)
	{
		checkAccess();
		if (priority < MIN_PRIORITY || priority > MAX_PRIORITY)
			throw new IllegalArgumentException();
		this.priority = Math.min(priority, group.getMaxPriority());

		NativeLang.setThreadPriority(this, priority);
	}

	public void run()
	{
		if (target != null)
			target.run();
	}

	public Thread(ThreadGroup group, Runnable target, String name)
	{
		Thread current = currentThread();

		if (group != null)
			group.checkAccess();
		else
			group = current.getThreadGroup();

		if (group == null) 
		{
			throw new InternalError("Live thread has invalid group: " +name);
		}

		group.add(this);

		this.group = group;
		this.target = target;
		this.name = name;

		this.priority = current.getPriority();
		this.daemon = current.isDaemon();
	}

	public Thread(ThreadGroup group, Runnable target)
	{
		this(group, target, autoName());
	}

	public Thread(ThreadGroup group, String name)
	{
		this(group, null, name);
	}

	public Thread(Runnable target, String name)
	{
		this(null, target, name);
	}

	public Thread(Runnable target)
	{
		this(null, target, autoName());
	}

	public Thread(String name)
	{
		this(null, null, name);
	}

	public Thread()
	{
		this(null, null, autoName());
	}
}
