
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.net;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Hashtable;
import java.util.Vector;
import java.util.Date;

public abstract class URLConnection
{ 
	private static ContentHandlerFactory factory;

	private static Hashtable handlers = new Hashtable();

	private static Class cObject;
	private static Class cContentHandler;

	private static boolean defaultUseCaches = true;
	private static boolean defaultAllowUserInteraction = true;
	private static Hashtable defaultProperties = new Hashtable();

	private final static String[] types = {
		"txt", "text/plain",
		"html", "text/html",
		"gif", "image/gif",
		"jpeg", "image/jpeg",
		"wrl", "model/vrml"
		// etc.
	};

	private static Hashtable extensionTypes = new Hashtable();

	static
	{
		for (int i = 0; i < types.length; i += 2)
			extensionTypes.put(types[i], types[i + 1]);
	}

	private static boolean isContentHandler(Class a)
	{
		try
		{
			if (cObject == null)
				cObject = Class.forName("java.lang.Object");
			if (cContentHandler == null)
				cContentHandler = Class.forName("java.net.ContentHandler");
		}
		catch (ClassNotFoundException e)
		{
		}

		while (true)
		{
			if (a.equals(cObject))
				return false;
			if (a.equals(cContentHandler))
				return true;
			a = a.getSuperclass();
		}
	}

	private static ContentHandler makeHandler(String type)
	{
		ContentHandler handler = null;

		if (factory != null)
			handler = factory.createContentHandler(type);

		if (handler != null)
			return handler;

		String name = "sun.net.www.content." + type.replace('/', '.');

		try
		{
			Class clazz = Class.forName(name);
			if (isContentHandler(clazz))
				handler = (ContentHandler) clazz.newInstance();
		}
		catch (ClassNotFoundException e)
		{
		}
		catch (InstantiationException e)
		{
		}
		catch (IllegalAccessException e)
		{
		}

		return handler;
	}

	private static ContentHandler getHandler(String type)
	{
		ContentHandler handler = (ContentHandler) handlers.get(type);
		if (handler != null)
			return handler;

		handler = makeHandler(type);
		if (handler != null)
			handlers.put(type, handler);

		return handler;
	}

	protected static String guessContentTypeFromName(String filename)
	{
		int dot = filename.lastIndexOf('.');
		String extension = filename.substring(dot + 1);
		String type = (String) extensionTypes.get(extension);
		return type;
	}

	protected static String guessContentTypeFromStream(InputStream is)
	{
		// TODO: implement a better guessing algorithm :)
		return "text/plain";
	}

	public static void setDefaultAllowUserInteraction(boolean flag)
	{
		defaultAllowUserInteraction = flag;
	}

	public static boolean getDefaultAllowUserInteraction()
	{
		return defaultAllowUserInteraction;
	}

	public static void setDefaultRequestProperty(String key, String value)
	{
		defaultProperties.put(key, value);
	}

	public static String getDefaultRequestProperty(String key)
	{
		return (String) defaultProperties.get(key);
	}

	public static void setContentHandlerFactory(ContentHandlerFactory fac)
	{
		if (factory != null)
			throw new Error();
		factory = fac;
	}

	private Hashtable headers = new Hashtable();
	private Vector headerValues = new Vector();
	private Vector headerKeys = new Vector();

	private Hashtable properties = new Hashtable();

	private ContentHandler handler;

	private InputStream in;
	private OutputStream out;

	protected boolean allowUserInteraction = defaultAllowUserInteraction;
	protected boolean useCaches = defaultUseCaches;
	protected boolean connected = false;
	protected boolean doInput = true;
	protected boolean doOutput = false;
	protected long ifModifiedSince = 0;
	protected URL url = null;

	public abstract void connect() throws IOException;

	public void setDefaultUseCaches(boolean flag)
	{
		defaultUseCaches = flag;
	}

	public boolean getDefaultUseCaches()
	{
		return defaultUseCaches;
	}

	public void setAllowUserInteraction(boolean flag)
	{
		allowUserInteraction = flag;
	}

	public boolean getAllowUserInteraction()
	{
		return allowUserInteraction;
	}

	public void setUseCaches(boolean flag)
	{
		useCaches = flag;
	}

	public boolean getUseCaches()
	{
		return useCaches;
	}

	public void setDoInput(boolean flag)
	{
		doInput = flag;
	}

	public boolean getDoInput()
	{
		return doInput;
	}

	public void setDoOutput(boolean flag)
	{
		doOutput = flag;
	}

	public boolean getDoOutput()
	{
		return doOutput;
	}

	public void setIfModifiedSince(long val)
	{
		ifModifiedSince = val;
	}

	public long getIfModifiedSince()
	{
		return ifModifiedSince;
	}

	public void setRequestProperty(String key, String value)
	{
		properties.put(key, value);
	}

	public String getRequestProperty(String key)
	{
		return (String) properties.get(key);
	}

	public URL getURL()
	{
		return url;
	}

	public InputStream getInputStream() throws IOException
	{
		return in;
	}

	public OutputStream getOutputStream() throws IOException
	{
		return out;
	}

	public long getDate()
	{
		return getHeaderFieldDate("Date", 0L);
	}

	public long getExpiration()
	{
		return getHeaderFieldDate("Expires", 0L);
	}

	public long getLastModified()
	{
		return getHeaderFieldDate("Last-Modified", 0L);
	}

	public int getContentLength()
	{
		return getHeaderFieldInt("Content-Length", 0);
	}

	public String getContentEncoding()
	{
		return getHeaderField("Content-Encoding");
	}

	public String getContentType()
	{
		return getHeaderField("Content-Type");
	}

	public Object getContent() throws IOException
	{
		String type = getContentType();

		if (handler == null)
			handler = getHandler(type);

		return handler.getContent(this);
	}

	private void addHeader(String key, String value)
	{
		headerKeys.addElement(key);
		headerValues.addElement(value);
		headers.put(key, value);
	}

	public String getHeaderField(int n)
	{
		if (n < headerValues.size())
			return (String) headerValues.elementAt(n);
		else
			return null;
	}

	public String getHeaderField(String name)
	{
		return (String) headers.get(name);
	}

	public long getHeaderFieldDate(String name, long def)
	{
		String str = getHeaderField(name);
		if (str == null)
			return def;
		Date date = new Date(str);
		return date.getTime() / 1000;
	}

	public int getHeaderFieldInt(String name, int def)
	{
		String str = getHeaderField(name);
		if (str == null)
			return def;
		return Integer.parseInt(str);
	}

	public String getHeaderFieldKey(int n)
	{
		if (n < headerKeys.size())
			return (String) headerKeys.elementAt(n);
		else
			return null;
	}

	public String toString()
	{
		return super.toString();
	}
	
	protected URLConnection(URL url)
	{
		this.url = url;
		useCaches = defaultUseCaches;
		allowUserInteraction = defaultAllowUserInteraction;
	}
}

