
// (C) 1996 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.util;

class ArrayEnumeration implements Enumeration
{
	protected Object[] data;
	protected int index;

	// Methods

	public boolean hasMoreElements()
	{
		return index < data.length;
	}

	public Object nextElement() throws NoSuchElementException
	{
		if (index >= data.length)
			throw new NoSuchElementException();
		return data[index++];
	}

	// Constructors

	ArrayEnumeration(Object[] data)
	{
		this.data = data;
		index = 0;
	}
}

public class Vector extends Object implements Cloneable
{ 
	// Fields

	protected int capacityIncrement;
	protected int elementCount;
	protected Object[] elementData;

	// Methods

	public final int capacity()
	{
		return elementData.length;
	}

	public final void ensureCapacity(int minCapacity)
	{
		if (minCapacity <= elementData.length)
			return;

		int newSize =
			capacityIncrement > 0
			? elementData.length + capacityIncrement
			: elementData.length * 2;

		if (newSize < minCapacity)
			newSize = minCapacity;

		Object[] newData = new Object[newSize];
		copyInto(newData);
		elementData = newData;
	}

	public final void copyInto(Object[] array)
	{
		System.arraycopy(
			(Object)elementData, 0,
			(Object)array, 0,
			elementCount
		);
	}

	public Object clone()
	{
		try
		{
			Vector v = (Vector) super.clone();
			v.elementData = (Object[]) elementData.clone();

			return v;
		}
		catch (CloneNotSupportedException e)
		{
			return null;
		}
	}

	public final void trimToSize()
	{
		Object[] newData = new Object[elementCount];
		copyInto(newData);
		elementData = newData;
	}

	public final void addElement(Object obj)
	{
		ensureCapacity(elementCount + 1);
		elementData[elementCount] = obj;
		elementCount++;
	}

	public final void insertElementAt(Object obj, int index)
	{
		if (index > elementCount)
			throw new IndexOutOfBoundsException();
		ensureCapacity(elementCount + 1);
		for (int i = elementCount; i > index; i--)
			     elementData[i] = elementData[i - 1];
		elementData[index] = obj;
		elementCount++;
	}

	public final int size()
	{
		return elementCount;
	}

	public final boolean isEmpty()
	{
		return (elementCount == 0);
	}

	public final Object firstElement()
		throws NoSuchElementException
	{
		if (isEmpty())
			throw new NoSuchElementException();
		return elementData[0];
	}

	public final Object lastElement()
		throws NoSuchElementException
	{
		if (isEmpty())
			throw new NoSuchElementException();
		return elementData[elementCount - 1];
	}

	public final Object elementAt(int index)
	{
		if (index >= elementCount)
			throw new IndexOutOfBoundsException();
		return elementData[index];
	}

	public final Enumeration elements()
	{
		Object[] data = new Object[elementCount];
		copyInto(data);
		return new ArrayEnumeration(data);
	}

	public final int indexOf(Object obj, int index)
	{
		for (int i = index; i < elementCount; i++)
			if (elementData[i].equals(obj))
				return i;
		return -1;
	}

	public final int indexOf(Object obj)
	{
		return indexOf(obj, 0);
	}

	public final boolean contains(Object obj)
	{
		return (indexOf(obj) >= 0);
	}

	public final int lastIndexOf(Object obj, int index)
	{
		int k = -1;
		for (int i = 0; i <= index; i++)
			if (elementData[i].equals(obj))
				k = i;
		return k;
	}

	public final int lastIndexOf(Object obj)
	{
		return lastIndexOf(obj, elementCount - 1);
	}

	public final void removeAllElements()
	{
		while (elementCount > 0)
			elementData[--elementCount] = null;
	}

	public final void removeElementAt(int index)
	{
		for (int i = index + 1; i < elementCount; i++)
			elementData[i - 1] = elementData[i];
		elementCount--;
	}

	public final boolean removeElement(Object obj)
	{
		for (int i = 0; i < elementCount; i++)
			if (elementData[i].equals(obj))
			{
				removeElementAt(i);
				return true;
			}
		return false;
	}

	public final void setElementAt(Object obj, int index)
	{
		if (index >= elementCount)
			throw new IndexOutOfBoundsException();
		elementData[index] = obj;
	}

	public final void setSize(int newSize)
	{
		while (elementCount > newSize)
			elementData[--elementCount] = null;
		ensureCapacity(newSize);
		while (elementCount < newSize)
			elementData[elementCount++] = null;
	}

	public final String toString()
	{
		StringBuffer buff = new StringBuffer("[");
		for (int i = 0; i < elementCount; i++)
		{
			if (i > 0)
				buff.append(", ");

			if (elementData[i] != null)
				buff.append(elementData[i].toString());
			else
				buff.append("null");
		}
		buff.append("]");

		return buff.toString();
	}
 
	// Constructors

	public Vector(int initialCapacity, int capacityIncrement)
	{
		this.capacityIncrement = capacityIncrement;
		elementData = new Object[initialCapacity];
		elementCount = 0;
	}
 
	public Vector(int initialCapacity)
	{
		this(initialCapacity, 0);
	}

	public Vector()
	{
		this(10);
	}
}

