
// (C) 1996 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package kore.util;

import java.util.Date;
import java.util.Vector;

public final class DateParser
{
	private final static String EOF = "EOF".intern();
	private final static String space = " ".intern();
	private final static String plus = "+".intern();
	private final static String minus = "-".intern();
	private final static String comma = ",".intern();
	private final static String slash = "/".intern();
	private final static String colon = ":".intern();

	private final static String GMT = "GMT".intern();
	private final static String UTC = "UTC".intern();
	private final static String UT = "UT".intern();

	private final static String EST = "EST".intern();
	private final static String CST = "CST".intern();
	private final static String MST = "MST".intern();
	private final static String PST = "PST".intern();

	private final static String EDT = "EST".intern();
	private final static String CDT = "CST".intern();
	private final static String MDT = "MST".intern();
	private final static String PDT = "PST".intern();

	private final static String AM = "AM".intern();
	private final static String PM = "PM".intern();

	private final static int TZ_UNKNOWN = Integer.MAX_VALUE;

	private String string;
	private int length;
	private int index = 0;

	private int tz		= TZ_UNKNOWN;
	private int year	= -1;
	private int month	= -1;
	private int day		= -1;
	private int hour	= -1;
	private int minute	= -1;
	private int second	= -1;

	private int read0()
	{
		return	(index >= length)
			? -1
			: (int) string.charAt(index++);
	}

	private void unread0()
	{
		index--;
	}

	private String getParentheses()
	{
		while (true)
		{
			int c = read0();
			if (c == -1)
				return EOF;
			if (c == ')')
				return space;
			if (c == '(')
				getParentheses();
		}
	}

	private String getSpace()
	{
		while (Character.isSpace((char) read0()))
			;
		unread0();
		return space;
	}

	private Integer getInt(int initial)
	{
		int value = initial - '0';

		while (true)
		{
			int c = read0();
			if (c < '0' || c > '9')
			{
				unread0();
				return new Integer(value);
			}
			value *= 10;
			value += c - '0';
		}
	}

	private String getWord(int initial)
	{
		StringBuffer buff = new StringBuffer();
		buff.append((char) initial);

		while (true)
		{
			int c = read0();
			if ((c < 'A' || c > 'Z') && (c < 'a' || c > 'z'))
			{
				unread0();
				return buff.toString().toUpperCase().intern();
			}
			buff.append((char) c);
		}
	}

	private Object getToken()
	{
		while (true)
		{
			int c = read0();
			switch (c)
			{
			case -1:	return EOF;
			case '+':	return plus;
			case '-':	return minus;
			case ',':	return comma;
			case '/':	return slash;
			case ':':	return colon;
			case '(':	return getParentheses();
			default:
				if (c >= '0' && c <= '9')
					return getInt(c);
				if (c >= 'A' && c <= 'Z')
					return getWord(c);
				if (c >= 'a' && c <= 'z')
					return getWord(c);
				if (Character.isSpace((char) c))
					return getSpace();
				throw new IllegalArgumentException(
					"Invalid character in date:" + new Character((char) c)
				);
			}
		}
	}

	private String[] days = {
		"SUNDAY", "MONDAY", "TUESDAY", "WEDNESDAY",
		"THURSDAY", "FRIDAY", "SATURDAY",
	};

	private boolean isDay(String s)
	{
		for (int i = 0; i < 7; i++)
			if (days[i].startsWith(s))
				return true;
		return false;
	}

	private String[] months = {
		"JANUARY", "FEBRUARY", "MARCH",
		"APRIL", "MAY", "JUNE",
		"JULY", "AUGUST", "SEPTEMBER",
		"OCTOBER", "NOVEMBER", "DECEMBER"
	};

	private int parseMonth(String s)
	{
		for (int i = 0; i < 12; i++)
			if (months[i].startsWith(s))
				return i;
		return -1;
	}

	public long parse()
	{
		Vector tokens = new Vector();
		while (true)
		{
			Object token = getToken();
			tokens.addElement(token);
			if (token == EOF)
				break;
		}

		int length = tokens.size();
		for (int i = 0; i < length; i++)
		{
			Object token = tokens.elementAt(i);
			if (token == EOF)
				break;

			Object next = tokens.elementAt(i + 1);

			if (token == GMT || token == UTC || token == UT)
			{
				tz = 0;
				continue;
			}

			if (token == plus || token == minus)
			{
				if (next instanceof Integer)
				{
					int n = ((Integer) next).intValue();
					tz = n;
					if (tz < 24)
						tz *= 60;
					else
						tz = (tz / 100) * 60 + (tz % 100);
					if (token == minus)
						tz = -tz;
					i++;
				}
				continue;
			}

			if (token instanceof Integer)
			{
				int n = ((Integer) token).intValue();

				if (n > 70 &&
				    next == space ||
				    next == comma ||
				    next == slash ||
				    next == EOF
				)
				{
					if (n > 1900)
						n -= 1900;
					year = n;
				}
				else if (next == colon)
				{
					if (hour < 0)
						hour = n;
					else
						minute = n;
					i++;
				}
				else if (next == slash)
				{
					if (month < 0)
						month = n - 1;
					else
						day = n;
					i++;
				}
				else if (
					 next == space ||
					 next == comma ||
					 next == minus ||
					 next == EOF
				)
				{
					if (hour >= 0 && minute < 0)
						minute = n;
					else if (minute >= 0 && second < 0)
						second = n;
					else
						day = n;
				}
				continue;
			}

			if (token instanceof String)
			{
				String word = (String) token;

				if (word == AM)
				{
					if (hour < 1 || hour > 12)
						throw new IllegalArgumentException("AM with invalid hour in date");
					continue;
				}

				if (word == PM)
				{
					if (hour < 1 || hour > 12)
						throw new IllegalArgumentException("PM with invalid hour in date");
					hour += 12;
					continue;
				}

				if (word == EDT)
				{
					tz = -240;
					continue;
				}

				if (word == EST || word == CDT)
				{
					tz = -300;
					continue;
				}

				if (word == CST || word == MDT)
				{
					tz = -360;
					continue;
				}

				if (word == MST || word == PDT)
				{
					tz = -420;
					continue;
				}

				if (word == PST)
				{
					tz = -480;
					continue;
				}

				if (isDay(word))
					continue;

				int m = parseMonth(word);
				if (m >= 0)
					month = m;
			}
		}

		if (year < 0)
			throw new IllegalArgumentException("no year in date");

		if (month < 0)
			throw new IllegalArgumentException("no month in date");

		if (day < 0)
			throw new IllegalArgumentException("no day of month in date");

		if (hour < 0)
			hour = 0;

		if (minute < 0)
			minute = 0;

		if (second < 0)
			second = 0;

		return (tz == TZ_UNKNOWN)
			? new Date(year, month, day, hour, minute, second).getTime()
			: (Date.UTC(year, month, day, hour, minute, second) - 60000 * tz);
	}

	public DateParser(String string)
	{
		this.string = string;
	}
}

