
// (C) 1997 Glynn Clements <glynn@sensei.co.uk> - Freely Redistributable

package java.awt;

import java.awt.peer.ComponentPeer;

import java.awt.image.ColorModel;
import java.awt.image.ImageObserver;
import java.awt.image.ImageProducer;

import java.io.PrintStream;

public abstract class Component implements ImageObserver
{
	static long delay;

	static
	{
		String inc = System.getProperty("awt.image.incrementalDraw");
		if (inc == null || inc.equals("true"))
		{
			String rate = System.getProperty("awt.image.redrawrate");
			delay = (rate == null) ? 100 : Integer.parseInt(rate);
		}
		else
			delay = -1;
	}

	ComponentPeer peer = null;
	Container parent = null;

	int x;
	int y;
	int width;
	int height;

	boolean valid = false;

	Color background = null;
	Color foreground = null;
	Font font = null;
	boolean enabled = true;
	boolean visible = true;

	protected String paramString()
	{
		return "bounds=" + bounds();
	}

	public String toString()
	{
		return super.toString();
	}

	public void addNotify()
	{
		valid = false;
	}

	public void removeNotify()
	{
		if (peer != null)
			peer.dispose();
		peer = null;
	}

	public boolean postEvent(Event e)
	{
		if (handleEvent(e))
			return true;
		if (parent != null && parent.handleEvent(e))
			return true;
		if (peer != null && peer.handleEvent(e))
			return true;
		return false;
	}

	public void deliverEvent(Event e)
	{
		postEvent(e);
	}

	public boolean inside(int x, int y)
	{
		return bounds().inside(x, y);
	}

	public Component locate(int x, int y)
	{
		return inside(x, y) ? this : null;
	}

	public void list(PrintStream out, int indent)
	{
		for (int i = 0; i < indent; i++)
			out.print(" ");
		out.println(this);
	}

	public void list(PrintStream out)
	{
		list(out, 0);
	}

	public void list()
	{
		list(System.out);
	}

	public void layout()
	{
	}

	public void invalidate()
	{
		valid = false;
	}

	public void validate()
	{
		if (valid)
			return;
		layout();
		valid = true;
	}

	public void nextFocus()
	{
		if (peer != null)
			peer.nextFocus();
	}

	public void requestFocus()
	{
		if (peer != null)
			peer.requestFocus();
	}

	public Image createImage(ImageProducer producer)
	{
		return	(peer != null)
			? peer.createImage(producer)
			: getToolkit().createImage(producer);
	}

	public Image createImage(int width, int height)
	{
		return createImage(width, height);
	}

	public boolean prepareImage(Image image, int width, int height, ImageObserver observer)
	{
		return	(peer != null)
			? peer.prepareImage(image, width, height, observer)
			: getToolkit().prepareImage(image, width, height, observer);
	}

	public boolean prepareImage(Image image, ImageObserver observer)
	{
		return prepareImage(image, -1, -1, observer);
	}

	public int checkImage(Image image, int width, int height, ImageObserver observer)
	{
		return	(peer != null)
			? peer.checkImage(image, width, height, observer)
			: getToolkit().checkImage(image, width, height, observer);
	}

	public int checkImage(Image image, ImageObserver observer)
	{
		return checkImage(image, -1, -1, observer);
	}

	public boolean imageUpdate(Image img, int flags, int x, int y, int w, int h)
	{
		if ((flags & ABORT) != 0)
			return false;
		if ((flags & ERROR) != 0)
			return false;
		else if ((flags & ALLBITS) != 0)
			repaint(delay, 0, 0, width, height);
		else if ((flags & SOMEBITS) != 0)
			repaint(delay, x, y, w, h);

		return (flags & ALLBITS) != 0;
	}

	public void disable()
	{
		enabled = false;
		if (peer != null)
			peer.disable();
	}

	public void enable()
	{
		enabled = true;
		if (peer != null)
			peer.enable();
	}

	public void enable(boolean flag)
	{
		if (flag)
			enable();
		else
			disable();
	}

	public boolean isEnabled()
	{
		return enabled;
	}

	public void hide()
	{
		visible = false;
		if (peer != null)
			peer.hide();
	}

	public void show()
	{
		visible = true;
		if (peer != null)
			peer.show();
	}

	public void show(boolean flag)
	{
		if (flag)
			show();
		else
			hide();
	}

	public boolean isVisible()
	{
		return visible;
	}

	public boolean isShowing()
	{
		return	isVisible() &&
			parent != null &&
			parent.isVisible() &&
			parent.isShowing();
	}

	public boolean isValid()
	{
		return valid;
	}

	public void move(int x, int y)
	{
		this.x = x;
		this.y = y;
		valid = false;
	}

	public void resize(int width, int height)
	{
		this.width  = width;
		this.height = height;
		valid = false;
	}

	public void resize(Dimension d)
	{
		resize(d.width, d.height);
	}

	public void reshape(int x, int y, int width, int height)
	{
		move(x, y);
		resize(width, height);
	}

	public Dimension minimumSize()
	{
		return	(peer != null)
			? peer.minimumSize()
			: size();
	}

	public Dimension preferredSize()
	{
		return	(peer != null)
			? peer.preferredSize()
			: size();
	}

	public Point location()
	{
		return new Point(x, y);
	}

	public Dimension size()
	{
		return new Dimension(width, height);
	}

	public Rectangle bounds()
	{
		return new Rectangle(x, y, width, height);
	}

	public void update(Graphics g)
	{
		g.clearRect(0, 0, width, height);
		g.setColor(foreground);
		paint(g);
	}

	public void paint(Graphics g)
	{
		if (peer != null)
			peer.paint(g);
	}

	public void paintAll(Graphics g)
	{
		paint(g);
	}

	public void print(Graphics g)
	{
		paint(g);
	}

	public void printAll(Graphics g)
	{
		print(g);
	}

	public void repaint(long tm, int x, int y, int width, int height)
	{
		repaint(tm, x, y, width, height);
	}

	public void repaint(int x, int y, int width, int height)
	{
		repaint(x, y, width, height);
	}

	public void repaint(long tm)
	{
		repaint(tm, 0, 0, width, height);
	}

	public void repaint()
	{
		repaint(0, 0, 0, width, height);
	}

	public void setBackground(Color c)
	{
		background = c;
	}

	public void setForeground(Color c)
	{
		foreground = c;
	}

	public void setFont(Font f)
	{
		font = f;
	}

	public Container getParent()
	{
		return parent;
	}

	public ComponentPeer getPeer()
	{
		return peer;
	}

	public Color getForeground()
	{
		return	(foreground != null)
			? foreground
			: (parent != null)
			  ? parent.getForeground()
			  : null;
	}

	public Color getBackground()
	{
		return	(background != null)
			? background
			: (parent != null)
			  ? parent.getBackground()
			  : null;
	}

	public Font getFont()
	{
		return	(font != null)
			? font
			: (parent != null)
			  ? parent.getFont()
			  : null;
	}

	public Graphics getGraphics()
	{
		return	(peer != null)
			? peer.getGraphics()
			: null;
	}

	public ColorModel getColorModel()
	{
		return	(peer != null)
			? peer.getColorModel()
			: getToolkit().getColorModel();
	}

	public FontMetrics getFontMetrics(Font font)
	{
		return	(peer != null)
			? peer.getFontMetrics(font)
			: getToolkit().getFontMetrics(font);
	}

	public Toolkit getToolkit()
	{
		return	(parent != null)
			? parent.getToolkit()
			: Toolkit.getDefaultToolkit();
	}

	public boolean handleEvent(Event e)
	{
		switch (e.id)
		{
		case Event.ACTION_EVENT:	return action(e, e.arg);
		case Event.GOT_FOCUS:		return gotFocus(e, null);
		case Event.LOST_FOCUS:		return lostFocus(e, null);
		case Event.KEY_ACTION:		return keyDown(e, e.key);
		case Event.KEY_ACTION_RELEASE:	return keyUp(e, e.key);
		case Event.KEY_PRESS:		return keyDown(e, e.key);
		case Event.KEY_RELEASE:		return keyUp(e, e.key);
		case Event.MOUSE_DOWN:		return mouseDown(e, e.x, e.y);
		case Event.MOUSE_UP:		return mouseUp(e, e.x, e.y);
		case Event.MOUSE_MOVE:		return mouseMove(e, e.x, e.y);
		case Event.MOUSE_DRAG:		return mouseDrag(e, e.x, e.y);
		case Event.MOUSE_ENTER:		return mouseEnter(e, e.x, e.y);
		case Event.MOUSE_EXIT:		return mouseExit(e, e.x, e.y);
		default:			return false;
		}
	}

	public boolean action(Event e, Object what)
	{
		return false;
	}

	public boolean gotFocus(Event e, Object what)
	{
		return false;
	}

	public boolean lostFocus(Event e, Object what)
	{
		return false;
	}

	public boolean keyDown(Event e, int key)
	{
		return false;
	}

	public boolean keyUp(Event e, int key)
	{
		return false;
	}

	public boolean mouseDown(Event e, int x, int y)
	{
		return false;
	}

	public boolean mouseUp(Event e, int x, int y)
	{
		return false;
	}

	public boolean mouseMove(Event e, int x, int y)
	{
		return false;
	}

	public boolean mouseDrag(Event e, int x, int y)
	{
		return false;
	}

	public boolean mouseEnter(Event e, int x, int y)
	{
		return false;
	}

	public boolean mouseExit(Event e, int x, int y)
	{
		return false;
	}
}
